/*
 * qmail-destroy: Remove messages from a qmail queue.
 * Copyright (C) 2015 LinuxMagic
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: queue-remove-util.h 19983 2015-10-15 22:39:44Z william $
 *
 * Utility functions.
 */

#ifndef __QUEUE_REMOVE_UTIL_H
#define __QUEUE_REMOVE_UTIL_H

#include <stdbool.h>
#include <stddef.h>

// wrapper macros
#define QRUERROR(...)  __QRUERROR(__FUNCTION__, __FILE__, __LINE__, __VA_ARGS__)
#define QRUDEBUG(...)  __QRUDEBUG(__FUNCTION__, __FILE__, __LINE__, __VA_ARGS__)

//! Method to find out if a given directory exists or not.
/*!
 * This also tests inode types so will return FALSE if it exists as
 * a file or a symlink. (POSIX only.. these concepts don't exist in win32...)
 *
 * @param   string  directory path to test
 *
 * @returns bool    exists or not
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 */
bool qru_directory_exists(const char * const);

//! Method to find out if given file exists or not.
/*!
 * @param   string  file path to test
 *
 * @returns bool    exists or not
 */
bool qru_file_exists(const char *);

//! Function to open a file in a common 'cross platform' way.
/*!
 * Using this function offers full compatibility
 * on both POSIX as well as windows for binary mode
 * file operations.
 *
 * @param   string      file path to open
 * @param   bool        flag to indicate if the file should be write locked on open
 * @param   integer     mode to open with (man fcntl...)
 *
 * @returns int fd or -1
 */
int qru_open(const char *, bool, int);

//! Function to safely close a file handle
/*!
 * This approach offers full compatibility
 * on both POSIX as well as windows systems.
 *
 * @param   int file handle to close
 *
 * @returns bool    success or fail
 */
bool qru_close(int);

//! Method to determine the current file position for a given handle.
/*!
 * @param   int the file handle to operate on
 *
 * @returns integer the current position from start (in bytes)
 */
int qru_tell(int);

//! Method to move the current file position to a given offset
/*!
 * @param   int the file handle to seek in
 * @param   integer the number of bytes to seek
 * @param   integer the starting point to seek from (see SEEK_SET, SEEK_CUR, SEEK_END)
 *
 * @returns bool    success or failure
 */
bool qru_seek(int, int, int);

//! Method to write a block of data to a given file handle
/*!
 * This is the compatibility equivalent of
 * fwrite and write functions. This method offers full
 * compatibility on both POSIX as well as windows systems.
 *
 * @param   int     the file handle to write to (MUST BE OPEN!)
 * @param   void*   the data to write out (void* since it can be any type)
 * @param   size_t  the size of the data to write out.
 *
 * @returns bool    success or fail.
 */
bool qru_write(int, void *, size_t);

//! Method to safely read a block of data from a file.
/*!
 * This will attempt to read up to given amount of data
 * from the file into the given destination memory space.
 * If the data available falls short, then so be it.
 *
 * @param   int     the file handle to read from (MUST BE OPEN!)
 * @param   void*   the destination buffer
 * @param   size_t  the maximum amount to read
 *
 * @returns bool    success or fail
 */
bool qru_read(int, void *, size_t);

//! utility function to set the 'run as' identity
/*!
 * @param   string  the username to run as
 *
 * @returns boolean success or fail of change.
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * NOTE: if the user running the program is the *SAME* as the
 *       set as then all good, otherwise it requires ROOT privileges
 *       or sticky bit user set on binary.
 */
bool qru_set_run_as(const char * const);

//! utility function to set the 'run as' identity: group
/*!
 * @param   string  the group to run as
 *
 * @returns boolean success or fail of change.
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * NOTE: if the user running the program is the *SAME* as the
 *       set as then all good, otherwise it requires ROOT privileges
 *       or sticky bit user set on binary.
 */
bool qru_set_run_as_group(const char * const);

//! method to append a NUMBER to a string as a string.
/*!
 * In the event the destination string is 'null' then
 * a new memory space is created. Otherwise the memory
 * space is resized dynamically.
 *
 * @param   char ** destination memory space pointer
 * @param   integer the number to append
 *
 * @returns boolean success or fail.
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 */
bool qru_icat(char **, long);

//! Utility function for converting signed numerics to a string
/*!
 * using 'long' type so be aware there may be different maximums in 64 bit
 * environments.  If the destiation memory pointer passed is NULL then a new
 * string is allocated. Be sure to 'free' it when you are done.
 *
 * @param   char**  destination memory pointer.
 * @param   integer the number to convert
 *
 * @returns bool    success or fail
 * @author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 */
bool qru_ltoa(char **, long);

//! Method for appending strings to each other.
/*!
 * In the event that the destination string is 'null' then
 * a new memory space is created.  Otherwise, the memory
 * space is resized dynamically.
 *
 * @param   char**  destination memory space pointer
 * @param   const char* source to append
 *
 * @returns bool    success or fail
 * @author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 */
bool qru_strcat(char **, const char *);

//! common function for logging information
/*!
 * @param char *format format string for the information to log
 *
 * This function will only log IF the logging
 * hooks have been defined via 'set_logger'
 */
void QRUINFO(const char *, ...);

//! common function for logging errors.
/*!
 * @param const char *fn the calling function that caused the error
 * @param const char *file the file that generated the error
 * @param int line number that the error occurred on
 * @param formatted message string printf() as an example
 * @param ... mixed arguments for the formatted message
 *
 * This function will only log IF the logging
 * hooks have been set via set_logger.
 */
void __QRUERROR(const char *, const char *, int, char *, ...);

//! common function for logging debug messages.
/*!
 * @param const char *fn the function that caused the debug
 * @param const char *file the file the error occurred in
 * @param int line the line number the error occurred on
 * @param const char *format the formatted message string see printf for example
 *
 * This function will only log IF the logging
 * hooks have been set via set_logger
 */
void __QRUDEBUG(const char *, const char *, int, const char *, ...);

#endif
