/*
 * qmail-destroy: Remove messages from a qmail queue.
 * Copyright (C) 2015 LinuxMagic
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: queue-remove.h 19983 2015-10-15 22:39:44Z william $
 *
 * Functions related to removal of messages from the queue.
 */

#ifndef __QUEUE_REMOVE_H
#define __QUEUE_REMOVE_H

//! remove a message from the queue.
/*!
 * @params char* Path to the queue directory. Typically this will be
 *   /var/qmail/queue
 * @params int Directory split count. This tells us how many directories
 *   are present in queue directories (0, 1, etc). Typically this is
 *   23 and is compiled into qmail (conf-split).
 * @params unsigned long Message id
 *   This is the inode number / filename associated with the message.
 *   Per qmail design it is unique to a message in the queue.
 *   Note this is unsigned long as that is what qmail uses internally.
 *   See fmtqfn() in qmail.
 *
 * @returns int Count of how many recipients flagged. -1 if an error
 *   occurred.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * This function will remove a message from the queue.
 *
 * The method involves flagging each recipient as done in the qmail
 * raw files.
 *
 * The files themselves are still present on disk after a return
 * from this function but will be cleaned up by qmail the next time it
 * runs a cleanup (TODO confirm where/when exactly).
 *
 * This means that the message will still show in the queue but it is
 * removed from the queue in that no further deliveries will
 * be attempted.
 *
 * Specific information about removal:
 * - For each message file containing recipients for the message (one
 *   may be in the local directory/channel and one in remote), open it.
 * - For each recipient in the file (format: T<email>\0T<email>\0...)
 *   replace each T with D to flag the recipient as done.
 * - Close the file!
 *
 * TODO:
 * - Possible race condition: Told to remove message id from the queue.
 *   Between then and now it is delivered and cleaned up. Between clean
 *   up and now a new message injected and set up for delivery. We then
 *   flag another message as done than the one intended.
 * - Separate function to trigger immediate clean up of the files? This
 *   may be done through the pulling of the "trigger" (see qmail/INTERNALS).
 */
int qmail_queue_remove(const char * const, const int,
    const unsigned long);

//! check if this message is in the queue.
/*!
 * @params char* Path to the queue directory. Typically this will be
 *   /var/qmail/queue
 * @params int Directory split count. This tells us how many directories
 *   are present in queue directories (0, 1, etc). Typically this is
 *   23 and is compiled into qmail (conf-split).
 * @params unsigned long Message id
 *   This is the inode number / filename associated with the message.
 *   Per qmail design it is unique to a message in the queue.
 *   Note this is unsigned long as that is what qmail uses internally.
 *   See fmtqfn() in qmail.
 *
 * @returns int 1 if it is, 0 if it is not, -1 if there is an error
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * this function checks if the mess file exists.
 *
 * it can be useful to know if there is a message with this id in the
 * queue in the first place - to be able to tell the difference between
 * no recipients found and message not in the queue.
 */
int qmail_is_message_in_queue(const char * const,
    const int, const unsigned long);

#endif
